//  RW Dollhouse
// ==============
// Docking, Charging-System, Storage, and General Outboard Systems Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/dollhouse/dollhouse.hpp
// -Declarations of the baseline RW Dollhouse library

#pragma once

#include "incls_all.hpp"

///////
// Dolldrop/Dollnet is meant for network-enabled projects only
#if __has_include(<curl_easy.h>)

// clang-format off
namespace rw::dollhouse::net  { class Dollnet; }
// clang-format on
//
using rw::dollhouse::net::Dollnet;

#endif  // __has_include(<curl_easy.h>)

namespace rw::dollhouse {

//------------------------------------------------------------------------------
class Dollhouse_b : public RW {
 public:
  Dollhouse_b();                                    // default ctor
  explicit Dollhouse_b(std::string const& handle);  // tag-parm'd ctor

 protected:
///////
// Dolldrop/Dollnet is meant for network-enabled projects only
#if __has_include(<curl_easy.h>)

  /////////////////
  //  'Net Work:
  //

  // DESIGN:
  // -As a simple, basic control mechanism for starters we'll use some
  // protected/private classes and a few functions from here within Dollhouse.
  // Later, we can add more access restrictions and some further data
  // validations; improving things bit-by-bit as we go along, using just a few
  // of the straightforward language features readily available to us.

  // DESIGN:
  // - add work-stealing queues of Dolldrop 's
  // - use S.Parent's libdispatch-based async
  // - this is actually a general need, so add it to the RW Foundations instead

  // DESIGN:
  // void push_drop() {}
  // void yank_drop() {}

  // DESIGN: TODO:
  // return do_atomiconn(
  //   lockout();
  //     connect();
  //       do_req(uri);
  //     disconnect();
  //   unlockout();
  // );

  class Dolldrop {
   public:
    Dolldrop(Hip const& tag) : owner_{tag} { dollnet_.bind_master(tag); }

    auto ask_drop(Hip const& tag, const std::string& uri)
        -> std::ostringstream {  //

      // TEST:
      prt_tag(tag, "asked to pull a drop");
      cout << "- requested pull drop : '" << uri << "'\n\n";

      // DESIGN: TODO: validate the Hip Type of 'tag' before proceeding
      // ...

      return dollnet_.dl_w_header(owner_, uri);
    }

    // TODO:
    auto get_drop(Hip const& tag) const -> std::string_view const {  //

      // TEST:
      prt_tag(tag, "asked for drop payload");

      return "foo_9001";  // TEMP:
    }

   private:
    class Inbox {
      //
     private:
      string data_;
    } inbox_;

    class Outbox {
      //
     private:
      string data_;
    } outbox_;

    Hip     owner_;
    Dollnet dollnet_{"dolldrop_dollnet"};

  } dolldrop{t()};  // construct dolldrop w/ this Dollhouse object as it's Owner

//
#endif  // __has_include(<curl_easy.h>)

};  //

//------------------------------------------------------------------------------
class Dollhouse : public Dollhouse_b {
 public:
  Dollhouse();                                    // default ctor
  explicit Dollhouse(std::string const& handle);  // tag-parm'd ctor

  // shell & power care
  void dock_me() {}
  void charge_me() {}
  void sleep_me() {}
  void wake_me() {}
  void store_me() {}

///////
// Dolldrop/Dollnet is meant for network-enabled projects only
#if __has_include(<curl_easy.h>)

  /** Attempts an external networking request to the supplied @a uri
   * @param[in] tag the caller's @c Hip
   * @param[in] uri the URI to attempt a download connection to
   * @return any header stream data for @a uri, if present
   */
  auto ask_drop(Hip const& tag, const std::string& uri) -> std::ostringstream {
    stamp(tag, "ask_drop()");

    return dolldrop.ask_drop(tag, uri);
  }

  auto get_drop(Hip const& tag) -> std::string_view const {
    stamp(tag, "get_drop()");

    return dolldrop.get_drop(tag);
  }

#endif  // __has_include(<curl_easy.h>)

  ///

  inline auto electro() const -> Electro& { return electro_; }
  inline auto shell() const -> Shell& { return shell_; }

 private:
  unique_ptr<Electro_b> _electro_{make_unique<Electro_b>("dollhouse_electro")};
  unique_ptr<Shell_b>   _shell_{make_unique<Shell_b>("dollhouse_shell")};

  Electro& electro_{reference_to<Electro>(_electro_.get())};
  Shell&   shell_{reference_to<Shell>(_shell_.get())};
};

}  // namespace rw::dollhouse

//    --=^=--

// "Now be careful. If you go for a full charge, it can get up there."

// rw_dollhouse_lib.v211218
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
