//   RW Dollnet
// ==============
// External, Inter-Networking Library
// -Part of the RW Foundations suite of libraries

// Filename: ./src/dollhouse/net/dollnet.cpp
// -Definitions of the baseline RW Dollnet sub-library

///////
// Dolldrop/Dollnet is meant for network-enabled projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

namespace rw::dollhouse::net {

//------------------------------------------------------------------------------

constexpr int tot_ticks{80};  // how wide the progress meter is (in chars)
Time          start;          // will start the instant before a DL begins

//------------------------------------------------------------------------------
/** Prints a progress meter out to the console
 * @param unused
 * @param total_dl
 * @param have_dl
 * @param unused
 * @param unused
 * @return
 */
int prog_func(Address,
              curl_off_t total_dl,
              curl_off_t have_dl,
              curl_off_t,
              curl_off_t) {  //

  // ensure download isn't empty (results in 'div by zero' error next statement)
  if (total_dl <= 0)
    return 0;

  // cast prevents integer division
  auto have_frac = have_dl / static_cast<float>(total_dl);

  // percent string
  auto          have_pct{have_frac * 100};
  ostringstream pct;
  pct << setw(6) << setfill(' ') << fixed << setprecision(1) << have_pct;

  // part we already have (the tickmarks)
  auto have = static_cast<int>(round(tot_ticks * have_frac));

  // part still remaining (the spaces)
  auto rem{tot_ticks - have};

  // tickmarks string
  // -(change the 'have' setfill() char if you'd like a different tick, Anon)
  ostringstream ticks;
  ticks << setw(have) << setfill('=') << "" << setw(rem) << setfill(' ') << "";

  // bytes string
  ostringstream m_of_n;
  m_of_n << setw(6) << have_dl << " of " << setw(6) << total_dl;

  // seconds string
  Secs_f        elapsed{Call_clk::now() - start};
  ostringstream secs;
  secs << setw(4) << setfill(' ') << fixed << setprecision(1)
       << elapsed.count();

  // everything's known & adjusted, now print progress meter to console
  cout << pct.str() << "% [" << ticks.str() << "]  bytes : " << m_of_n.str()
       << ",  in " << secs.str() << " secs\r";  // ( <- note the '\r' )

  // printing w/ flushing works smoother in this particular type of case ('\r')
  cout << flush;

  // finally, transfer will abort w/o a '0' func ret
  return 0;
}

//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
Dollnet::Dollnet() : Dollnet{"dollnet_obj"} {}

//------------------------------------------------------------------------------
Dollnet::Dollnet(std::string const& handle)
    : RW_slave{handle, {Type::Dollhouse}} {
  stamp("Dollnet()");
}

//------------------------------------------------------------------------------
auto Dollnet::dl_w_header(Hip const& tag, std::string const& uri)
    -> std::ostringstream {
  stamp(tag, "dl_w_header()");

  if (tag == master()) {  //

    // create a standard output file
    ofstream anons_file{};
    anons_file.open("./dolldrops/anons_file");

    // the "writer" stream will put the body's contents into "anons_file"
    curl_ios<ostream> writer{anons_file};

    // pass that stream to the Easy object's constructor for writing to disk
    curl_easy easy{writer};

    // we'll return this header data stream at the end of this code block
    ostringstream header_dat{};

    // the "header" stream will put the header's contents into "header_dat"
    curl_ios<ostringstream> header{header_dat};

    // we'll use curlcpp's default write function callback
    easy.add<CURLOPT_WRITEFUNCTION>(header.get_function());

    // specify the stream for the header's content
    easy.add<CURLOPT_HEADERDATA>(header.get_stream());

    // r/n we're simply known as 'Bumpmaster' to the rest of the world
    easy.add<CURLOPT_USERAGENT>("Bumpmaster_v211218");

    // we'll output our DL progress to the console
    easy.add<CURLOPT_XFERINFOFUNCTION>(prog_func);

    // yes, do go ahead and track our DL progress
    easy.add<CURLOPT_NOPROGRESS>(0L);

    // yes, follow locations
    easy.add<CURLOPT_FOLLOWLOCATION>(1L);

    // and here's where we're going
    easy.add<CURLOPT_URL>(uri.c_str());

    try {  //

      start = Call_clk::now();  // start the timer,
      easy.perform();           // and let's go!

    } catch (curl_easy_exception const& e) {  // oh noes!11 did it broke?

      // prints the last error
      cerr << "in Dollnet::dl_w_header()::try{} :\n" << e.what() << '\n';

      // prints the entire error stack
      e.print_traceback();
    }

    anons_file.close();  // _ALWAYS_ close output files Anon!  :^)
    return header_dat;

  } else
    only_master_can(master(), tag, t(), "Dollnet::dl_w_header()");

  ///
  return ostringstream{};  // shouldn't reach here
}

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "This apartment has super-high bandwidth illegal even to large corporations?"

// rw_dollhouse_lib.v211218
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
