//  RW Foundations
// ================
// The RW Foundational Libraries

// Filename: ./src/srcs_all.cpp
// -The following lines pull in the project's real *.cpp files.
// -note: this file also acts as a location for a few utility features

#include "rw/rw_srcs_all.cpp"
//
#include "action/action_srcs_all.cpp"
#include "animeyes/animeyes_srcs_all.cpp"
#include "arigato/arigato_srcs_all.cpp"
#include "atashi/atashi_srcs_all.cpp"
#include "bones/bones_srcs_all.cpp"
#include "chibitsu/chibitsu_srcs_all.cpp"
#include "curator/curator_srcs_all.cpp"
#include "dollhouse/dollhouse_srcs_all.cpp"
#include "electro/electro_srcs_all.cpp"
#include "gears/gears_srcs_all.cpp"
#include "graphicom/graphicom_srcs_all.cpp"
#include "hearsay/hearsay_srcs_all.cpp"
#include "meido/meido_srcs_all.cpp"
#include "shell/shell_srcs_all.cpp"

//------------------------------------------------------------------------------

#include "incls_all.hpp"

//------------------------------------------------------------------------------
void prt_tag(Hip const& tag, std::string const& msg) {
  Hdl_id const hid{tag};

  auto const f{cout.flags()};  // store flags state

  cout << setw(32) << msg << " - " << setw(24) << dec << hid
       << "    \taddr:  " << setw(16) << hex << tag.ptr << '\n';

  cout.flags(f);  // restore flags state
}

//------------------------------------------------------------------------------
void analyze_callgraph() {  //

  ///
  // load Callgraph:

  Callgraph cgraph{};

  for (auto const prw : RW_objs) {
    auto const rw_obj{reference_to<RW>(prw)};
    cgraph.emplace_back(RW_stamps{rw_obj.t(), rw_obj.get_stamps()});
  }

  ///
  // print Callgraph's items to console:

  cout << "Callgraph size (ie, the number of active RW objects) : "
       << cgraph.size() << "\n\n";

  for (auto& [out_tag, stamps] : cgraph) {  //
    Hdl_id const out_hid{out_tag};

    cout << " - Object's Handle.ID : '" << out_hid << "'\n";
    cout << "\tobject's stampstack size : " << stamps.size() << "\n";

    while (! stamps.empty()) {  //

      auto const& [in_tag, func_nm, tim]{stamps.top()};
      Hdl_id const in_hid{in_tag};

      cout << setw(24) << in_hid << '\t' << setw(6)
           << ((in_hid == out_hid) ? "(self)" : "")
           << "  calls obj's :" << setw(22) << func_nm << " func at :  " << tim
           << "  utc\n";

      stamps.pop();
    }

    puts("");
  }
}

//    --=^=--

// "So is it true Sempai? Are robowaifus really better than any human woman?"

// rw_foundations_lib.v211218
// Copyright (2021)
// License MIT(Expat) https://opensource.org/licenses/MIT
